local PETS_PER_TEAM = 3
local MAX_TEAMS = 60
local ROW_HEIGHT = 44.3   
local MAX_ROWS = 24      
PetBattleTeamsUI = {}

PetBattleTeamsUI.CursorEmpty = function(self)
	PetBattleTeamsUI.PetOnCursor = nil
	PetBattleTeamsUI.CursorStruct = nil
end



--update functions
PetBattleTeamsUI.UpdateAllFrames = function(self,msg)
	
	if self.mainFrame then
		self:UpdateAddTeamButton(self.mainFrame.addTeamButton,#PetBattleTeamsSettings.teams)
		self:UpdateMainFrame(self.mainFrame,#PetBattleTeamsSettings.teams)
		self:UpdateReviveButton(self.mainFrame.revivePetsButton, #PetBattleTeamsSettings.teams)
		self:UpdateScrollBar(msg)
	end
end

PetBattleTeamsUI.UpdateAddTeamButton = function(self,button,numTeams)
	if button then
		button:SetPoint("BOTTOMLEFT", button:GetParent() ,"BOTTOMLEFT",18,10)			
		if numTeams == MAX_TEAMS then 
			button.icon:SetDesaturated(true)	
		else
			button.icon:SetDesaturated(false)
		end
	end
end

PetBattleTeamsUI.UpdateReviveButton = function(self,button,numTeams)
	if button then
		local attached , locked = PetBattleTeamsSettings:GetFrameInfo()
		if not attached  then
			button:Show()
			button:SetPoint("BOTTOMRIGHT", button:GetParent() ,"BOTTOMRIGHT",-32,10)
		else
			button:Hide()
		end
	end
end

PetBattleTeamsUI.UpdateMainFrame=function(self,frame, numTeams)
	if frame then
		local attached , locked = PetBattleTeamsSettings:GetFrameInfo()
		
		
		if C_PetBattles.IsInBattle() or C_PetBattles.GetPVPMatchmakingInfo() then
			frame.GotoTeamButton.icon:SetTexture("Interface\\PetBattles\\PetBattle-LockIcon")
			frame.GotoTeamButton.icon:SetTexCoord(0,1,0,1)
		else
			frame.GotoTeamButton.icon:SetTexture("Interface\\PetBattles\\PetJournal")
			frame.GotoTeamButton.icon:SetTexCoord(0.11328125,0.16210938,0.02246094,0.04687500)
		end
		
		
		
		--set min and max sizes depending on number of teams
		if #PetBattleTeamsSettings.teams > 4 then 
			frame:SetMinResize(frame:GetWidth(),ROW_HEIGHT*4+90)
			if #PetBattleTeamsSettings.teams <= 12 then
				frame:SetMaxResize(frame:GetWidth(),ROW_HEIGHT*#PetBattleTeamsSettings.teams+90)
			else
				frame:SetMaxResize(frame:GetWidth(),ROW_HEIGHT*12+90)
			end
			frame.resizeButton:Show()
		else
			frame:SetMinResize(frame:GetWidth(),ROW_HEIGHT*1+90)
			frame:SetMaxResize(frame:GetWidth(),ROW_HEIGHT*#PetBattleTeamsSettings.teams+90)
			frame.resizeButton:Hide()
		end
		
		if attached and PetJournal then
			locked = false
			frame:SetParent(PetJournal)
			frame:ClearAllPoints()
			frame:SetPoint("TOPLEFT",PetJournal,"TOPRIGHT",-5,4)
		else
			frame:SetParent("UIParent")
		end
		
		if locked or attached then 
			frame:SetMovable(false)
			frame:SetScript("OnDragStart", nil)
			frame:SetScript("OnDragStop", nil)
		else
			frame:SetScript("OnDragStart", frame.StartMoving)
			frame:SetScript("OnDragStop", frame.StopMovingOrSizing)
			frame:SetMovable(true)
			--frame.menu:SetMovable(true)
		end
		
		if frame then
			if attached then
				frame:SetHeight(614)
				frame.resizeButton:Hide()
			else
				--handles frame sizing when you have less then minimum teams to activate the scroll bar
				if #PetBattleTeamsSettings.teams <= 4 then
					PetBattleTeamsUI.mainFrame:SetHeight(ROW_HEIGHT * #PetBattleTeamsSettings.teams +90)
				end
				
				--if we delete a team when the scroll bar is hidden then the pet frame is bigger then it should be
				if self.scrollBar and not self.scrollBar:IsShown() and (PetBattleTeamsUI.mainFrame:GetHeight()-90)/ROW_HEIGHT > #PetBattleTeamsSettings.teams then
					PetBattleTeamsUI.mainFrame:SetHeight(ROW_HEIGHT * #PetBattleTeamsSettings.teams +90)
				end
			end
		end
	end
end

PetBattleTeamsUI.UpdatePetButton=function(self,button,team,petIndex,msg)
	button:Hide()
	local lockedTeam = C_PetBattles.IsInBattle() or C_PetBattles.GetPVPMatchmakingInfo()
	
	if PetBattleTeamsSettings.teams[team] and PetBattleTeamsSettings.teams[team][petIndex] then
		
		button.team = team
		button.petIndex = petIndex
		button.petID = PetBattleTeamsSettings.teams[team][petIndex].petID
		
		button:Show()
		button.BorderAlive:Show()
		
		button.rarityGlow:Hide()
		button.selected:Hide()
		button.ActualHealthBar:Hide()
		button.level:Hide()
		button.BorderDead:Hide()
		
		--set icon for unknown pet
		button.Icon:SetTexture("\Interface\\ICONS\\INV_Misc_QuestionMark")
		
		if petIndex == 1 and PetBattleTeamsSettings.selected == team then
			button.selected:Show()
			if lockedTeam then
				button.selected:SetTexture("Interface\\PetBattles\\PetBattle-LockIcon")
				button.selected:SetTexCoord(0,1,0,1)
			else
				button.selected:SetTexture("Interface\\PetBattles\\PetJournal")
				button.selected:SetTexCoord(0.11328125,0.16210938,0.02246094,0.04687500)
			end
		end
		
		if lockedTeam and PetBattleTeamsSettings.selected == team then
			button:Disable()
		else
			button:Enable()
		end
		
		
		
		if button.petID and button.petID > 0 and C_PetJournal.GetPetInfoByPetID(button.petID) then
			-- get info about the pet
			local speciesID, customName, level, xp, maxXp, displayID, name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable = C_PetJournal.GetPetInfoByPetID(button.petID)
			local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(button.petID)
			
			if rarity then
				local r, g, b = GetItemQualityColor(rarity-1)
				button.rarityGlow:SetVertexColor(r, g, b)
			end
			
			button.level:SetText(level)
			button.level:Show()

			button.Icon:SetTexture(icon)

			--set health bar
			if health and maxHealth then
				if  ( health == 0 ) then
					button.ActualHealthBar:Hide()
				else
					button.ActualHealthBar:Show()
				end
				button.ActualHealthBar:SetWidth((health / max(maxHealth,1)) * button.healthBarWidth)
			end
			
			--Set alive border
			if health and ( health == 0 ) then
				button.BorderAlive:Hide()
				button.rarityGlow:Hide()
			else
				button.BorderAlive:Show()
				button.rarityGlow:Show()
			end
			
			--set dead border
			if health and ( health == 0 ) then
				button.BorderDead:Show()
				button.rarityGlow:Hide()
			else
				button.BorderDead:Hide()
				button.rarityGlow:Show()
			end
			
			for _, object in pairs(button.hideWhenDeadList) do
				if ( health == 0 ) then
					object:Hide()
				else
					object:Show()
				end
			end
			
			--button.Icon:SetVertexColor(1,1,1,1)
			local PetOnCursor = PetBattleTeamsUI.PetOnCursor
			local CursorStruct = PetBattleTeamsUI.CursorStruct
			button.Icon:SetVertexColor(1,1,1)
			--print(PetOnCursor,CursorStruct)
			button.Icon:SetDesaturated(false)
			
			
			
			local r,g,b = unpack(button.ActualHealthBar.color)
			button.ActualHealthBar:SetVertexColor(r,g,b)
			if PetOnCursor then
				
				if CursorStruct then
					
					
					if CursorStruct.sourceTeam ~= button.team and button.petID ~= PetOnCursor or (lockedTeam and PetBattleTeamsSettings.selected == button.team) then
						for i=1,PETS_PER_TEAM do
							if PetOnCursor == PetBattleTeamsSettings.teams[team][i].petID or button.petID == PetBattleTeamsSettings.teams[CursorStruct.sourceTeam][i].petID then
								--button.Icon:SetVertexColor(1,0,0)
								button.Icon:SetDesaturated(1)
								button.ActualHealthBar:SetVertexColor(.6,.6,.6)
								button.rarityGlow:Hide()
							end
						end
					end
				
				else
					
					if PetOnCursor ~= button.petID or (lockedTeam and PetBattleTeamsSettings.selected == button.team) then
						
						for i=1,PETS_PER_TEAM do
							if PetOnCursor == PetBattleTeamsSettings.teams[team][i].petID then
								--button.Icon:SetVertexColor(1,0,0)
								button.Icon:SetDesaturated(1)
								button.ActualHealthBar:SetVertexColor(.6,.6,.6)
								button.rarityGlow:Hide()
							end
						end
					end
				end
			end
			
			if lockedTeam and PetBattleTeamsSettings.selected == button.team then
				button.Icon:SetDesaturated(1)
				button.ActualHealthBar:SetVertexColor(.6,.6,.6)
				button.rarityGlow:Hide()
			end	
			
			
			
		end
	end
end

--tooltip update
PetBattleTeamsUI.TooltipSetUnit = function(self,tooltip, petID, team, petIndex)
	local speciesID, customName, level, xp, maxXp, displayID, petName, petIcon, petType, creatureID = C_PetJournal.GetPetInfoByPetID(petID)
	local pet = PetBattleTeamsSettings.teams[team][petIndex]
	
	local health, maxHealth, attack, speed, rarity = C_PetJournal.GetPetStats(petID)
	local r, g, b,hex = GetItemQualityColor(rarity-1)
	
	tooltip.rarityGlow:SetVertexColor(r, g, b)
	tooltip.Icon:SetTexture(petIcon)
	tooltip.Name:SetText("|c"..hex..petName.."|r")
	tooltip.Level:SetText(level)
	tooltip.XPBar:SetWidth(max((xp / max(maxXp,1)) * tooltip.xpBarWidth, 1))
	tooltip.Delimiter:SetPoint("TOP", tooltip.XPBG, "BOTTOM", 0, -10)
	tooltip.XPText:SetFormattedText(tooltip.xpTextFormat or PET_BATTLE_CURRENT_XP_FORMAT, xp, maxXp)
	tooltip.teamText:SetText("Team "..team)
	tooltip.AttackAmount:SetText(attack)
	tooltip.SpeedAmount:SetText(speed)
	tooltip.PetType.Icon:SetTexture("Interface\\PetBattles\\PetIcon-"..PET_TYPE_SUFFIX[petType])
	
	tooltip.SpeciesName:Hide()
	if customName then
		if ( customName ~= petName ) then
			tooltip.Name:SetText(customName)
			tooltip.SpeciesName:SetText(petName)
			tooltip.SpeciesName:Show()
		end
	end
	
	if ( tooltip.HealthText ) then
		tooltip.HealthText:SetFormattedText(tooltip.healthTextFormat or PET_BATTLE_CURRENT_HEALTH_FORMAT, health, maxHealth)
	end
	
	if ( health == 0 ) then
		tooltip.ActualHealthBar:SetWidth(1)
	else
		tooltip.ActualHealthBar:SetWidth((health / max(maxHealth,1)) * tooltip.healthBarWidth)
	end
		
	for i=1, #pet.abilities do
		local name, icon, petType = C_PetJournal.GetPetAbilityInfo(pet.abilities[i])
		local abilityIcon = tooltip["AbilityIcon"..i]
		local abilityName = tooltip["AbilityName"..i]

		abilityName:SetShown(name)
		abilityIcon:SetShown(icon)
		abilityIcon:SetTexture("Interface\\PetBattles\\BattleBar-AbilityBadge-Neutral")
		if name then 
			abilityName:SetText(name.." "..pet.abilities[i])
		end
	end
end

--Create UI functions
PetBattleTeamsUI.DropDownOnLoad=function(self,level)
	local info = UIDropDownMenu_CreateInfo()
	info.isNotRadio = true
	
	info.text = "Attach to Pet Journal"
	info.func = function(_, _, _, value) 
		local attached , locked = PetBattleTeamsSettings:GetFrameInfo()
		
		if not value then
			PetBattleTeamsSettings:SetFrameInfo(true, false)
		else
			PetBattleTeamsSettings:SetFrameInfo(false, locked)
			
			if #PetBattleTeamsSettings.teams <= 12 then
				PetBattleTeamsUI.mainFrame:SetHeight(ROW_HEIGHT * #PetBattleTeamsSettings.teams +90)
			end
		end
		PetBattleTeamsUI:UpdateAllFrames()
	end
	info.checked = function() return select(1,PetBattleTeamsSettings:GetFrameInfo()) end
	UIDropDownMenu_AddButton(info, level)
	
	info.text = "Lock Location"
	info.func = function(_, _, _, value)  
		local attached , locked = PetBattleTeamsSettings.GetFrameInfo()
		PetBattleTeamsSettings:SetFrameInfo(attached, not value)
		PetBattleTeamsUI:UpdateAllFrames()
	end
	info.checked = function()  return select(2,PetBattleTeamsSettings:GetFrameInfo()) end
	UIDropDownMenu_AddButton(info, level)
	
	info.notCheckable = true
	info.text = "Close Menu"
	info.func = nil
	UIDropDownMenu_AddButton(info, level)
end

PetBattleTeamsUI.CreateReviveButton =function(self,name,parent)
	local HEAL_PET_SPELL = 125439
	local spellName, spellSubname, spellIcon = GetSpellInfo(HEAL_PET_SPELL)
	local start, duration, enable = GetSpellCooldown(HEAL_PET_SPELL)
	
	button = CreateFrame("Button",name,parent,"secureactionbuttontemplate")
	button:SetAttribute("type", "spell")
	button.spellID = HEAL_PET_SPELL
	button:SetAttribute("spell",spellName)
	button:SetSize(40,40)
	
	button.Icon = button:CreateTexture(name.."Icon","ARTWORK")
	button.Icon:SetTexture(spellIcon)
	button.Icon:SetAllPoints()
	
	button.Border = button:CreateTexture(name.."Border","OVERLAY","ActionBarFlyoutButton-IconFrame")
	button:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")
	
	button.Cooldown = CreateFrame("Cooldown", name.."Cooldown",button, "CooldownFrameTemplate")
	CooldownFrame_SetTimer(button.Cooldown, start, duration, enable)
	
	button:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	
	button:SetScript("OnEvent", function(self,event,...) 
		if event == "SPELL_UPDATE_COOLDOWN"  then
			local start, duration, enable = GetSpellCooldown(self.spellID)
			CooldownFrame_SetTimer(self.Cooldown, start, duration, enable)
			if ( GameTooltip:GetOwner() == self ) then
				--cheat and use blizzards tooltip setup
				PetJournalHealPetButton_OnEnter(self)
			end
		end
	end)
	
	button:SetScript("OnShow", function(self) 
		self:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	end)
	button:SetScript("OnHide",function(self) 
		self:UnregisterEvent("SPELL_UPDATE_COOLDOWN")
	end)
	
	button:SetScript("OnEnter", PetJournalHealPetButton_OnEnter )
	button:SetScript("OnLeave",function()  GameTooltip:Hide() end)
	return button
end

PetBattleTeamsUI.CreateTooltip = function(self,name)
	local tooltip = CreateFrame("Frame",name,nil,"PetBattleUnitTooltipTemplate")
	tooltip:SetHeight(211)
	
	--icon quality glow
	tooltip.rarityGlow = tooltip:CreateTexture("PetBattleTeamTooltipGlow","OVERLAY")
	tooltip.rarityGlow:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
	tooltip.rarityGlow:SetBlendMode("ADD")
	tooltip.rarityGlow:ClearAllPoints()
	tooltip.rarityGlow:SetDrawLayer("OVERLAY", 0)
	tooltip.rarityGlow:SetWidth(tooltip.Icon:GetWidth() * 1.7)
	tooltip.rarityGlow:SetHeight(tooltip.Icon:GetHeight() * 1.7)
	tooltip.rarityGlow:SetPoint("CENTER", tooltip.Icon, "CENTER", 0, 0)
	
	--team indicator
	tooltip.teamText = tooltip:CreateFontString("PetBattleTeamTooltipTeamText","OVERLAY","GameFontNormalSmall")
	tooltip.teamText:SetJustifyH("CENTER")
	tooltip.teamText:SetText("Team XX")
	tooltip.teamText:SetTextColor(1,1,1)
	tooltip.teamText:SetSize(0,0)
	tooltip.teamText:SetFont("Fonts\\FRIZQT__.TTF",12,"OUTLINE")
	tooltip.teamText:SetPoint("BOTTOMLEFT",tooltip.Name,"TOPLEFT",0,2)

	--template parts
	tooltip.AbilitiesLabel:Show()
	tooltip.XPBar:Show()
	tooltip.XPBG:Show()
	tooltip.XPBorder:Show()
	tooltip.XPText:Show()
	tooltip.teamText:Show()
	tooltip.WeakToLabel:Hide()
	tooltip.ResistantToLabel:Hide()
	
	return tooltip
end

PetBattleTeamsUI.CreateMenuButton = function(self,name,parent)
	local button = CreateFrame("BUTTON",name,parent)
	button.menu = CreateFrame("Frame",name,frame,"UIDropDownMenuTemplate")
	button.menu:SetPoint("TOPRIGHT",button,"BOTTOMLEFT",0,0)

	UIDropDownMenu_Initialize(button.menu, PetBattleTeamsUI.DropDownOnLoad, "MENU")

	button:EnableMouse(true)
	button:SetSize(33,33)
	button:ClearAllPoints()
	
	button.icon = button:CreateTexture("PetBattleTeambuttonButtonIcon","ARTWORK")
	button.icon:SetTexture("Interface\\Icons\\INV_PET_BATTLEPETTRAINING")
	button.icon:SetSize(21,21)
	button.icon:ClearAllPoints()
	button.icon:SetPoint("TOPLEFT",button,"TOPLEFT",7,-6)
	
	button.overlay = button:CreateTexture("PetBattleTeambuttonButtonIcon","OVERLAY")
	button.overlay:SetTexture("Interface\\MiniMap\\MiniMap-TrackingBorder")
	button.overlay:SetSize(56,56)
	button.overlay:ClearAllPoints()
	button.overlay:SetPoint("TOPLEFT",button,"TOPLEFT")
	
	button:SetHighlightTexture("Interface\\MiniMap\\UI-MiniMap-ZoomButton-Highlight","ADD")
	button:RegisterForClicks("LeftButtonUp","RightButtonUp")
	button:SetScript("OnClick", function(self,button) 		
			ToggleDropDownMenu(1, nil, self.menu, self,20,7)
	end)
	return button
end

PetBattleTeamsUI.CreateParentFrame = function(self,name)
	local frame = CreateFrame("Frame", name, UIParent)
	frame:SetClampedToScreen(true)
	frame:SetFrameStrata("MEDIUM")
	frame:SetToplevel(true) 
	frame:SetSize(181,614)
	frame:RegisterForDrag("LeftButton")
	frame:EnableMouse(true)
	frame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = false, tileSize = 32, edgeSize=32, insets ={left=8,right=8,top=8,bottom=8}})
	frame:SetBackdropColor(.1,.1,.1,1)
	
	frame:SetMaxResize(frame:GetWidth(),frame:GetHeight())
	frame:SetMinResize(frame:GetWidth(),ROW_HEIGHT*1+90)
	
	--frame label
	frame.name = frame:CreateFontString(name.."Title","OVERLAY","GameFontNormal")
	frame.name:SetJustifyH("CENTER")
	frame.name:SetText("Pet Battle Teams")
	frame.name:SetSize(0,0)
	frame.name:SetPoint("CENTER",frame,"TOP",0,-15)
	
	--upper right menu button
	frame.menu = self:CreateMenuButton(name.."Menu",frame)
	frame.menu:SetParent(frame)
	frame.menu:ClearAllPoints()
	frame.menu:SetPoint("TOPRIGHT",frame,"TOPRIGHT",8,8)

	frame.addTeamButton = self:CreateAddTeamButton("PetBattleTeamAddTeamButton",frame)
	frame.addTeamButton:Show()
	frame.petUnitFrames = self:CreatePetUnitFrames(frame)
	frame.revivePetsButton = self:CreateReviveButton("PetBattleTeamReviveButton",frame)
	
	frame.resizeButton = CreateFrame("Button",nil,frame)
	frame.resizeButton:EnableMouse(true)
	frame.resizeButton:SetSize(20,20)
	frame.resizeButton:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",-6,6)
	frame.resizeButton.icon = frame.resizeButton:CreateTexture(nil,"ARTWORK")
	frame.resizeButton.icon:SetTexture("Interface\\CHATFRAME\\UI-ChatIM-SizeGrabber-Up")
	frame.resizeButton.icon:SetAllPoints(frame.resizeButton)
	frame.resizeButton:SetHighlightTexture("Interface\\CHATFRAME\\UI-ChatIM-SizeGrabber-Highlight","ADD")
	frame.resizeButton:SetPushedTexture("Interface\\CHATFRAME\\UI-ChatIM-SizeGrabber-Down")
	frame.resizeButton:RegisterForDrag("LeftButton","RightButton")
	--frame.resizeButton:RegisterForClicks("LeftButtonUp","RightButtonUp")
	
	frame.GotoTeamButton = CreateFrame("Frame",nil,frame);
	frame.GotoTeamButton:SetSize(36,36);
	frame.GotoTeamButton:SetPoint("CENTER",frame,"BOTTOM",-6,30)
	frame.GotoTeamButton.icon = frame.GotoTeamButton:CreateTexture(nil,"OVERLAY") 
	frame.GotoTeamButton.icon:SetSize(36,36)
	frame.GotoTeamButton.icon:SetTexture("Interface\\PetBattles\\PetJournal")
	frame.GotoTeamButton.icon:SetTexCoord(0.11328125,0.16210938,0.02246094,0.04687500)
	frame.GotoTeamButton.icon:SetAllPoints(frame.GotoTeamButton)
	frame.GotoTeamButton:EnableMouse(true)
	
	
	--got team button
	frame.GotoTeamButton:SetScript("OnMouseUp",function(self) 
		PetBattleTeamsUI:ScrollToTeam(PetBattleTeamsSettings.selected)
	end)
	
	frame.GotoTeamButton:SetScript("OnEnter",function(self)
		GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT")
		GameTooltip:ClearLines()
		
		GameTooltip:AddLine("Click to Scroll to your current team")
		
		GameTooltip:Show()
	end)
	frame.GotoTeamButton:SetScript("OnLeave",function(self)
		GameTooltip:Hide()
	end)
	
	
	
	--resizing
	frame:SetScript("OnSizeChanged",function(self) 
		PetBattleTeamsUI:UpdateAllFrames()
	end)
	
	frame.resizeButton:SetScript("OnDragStart",function(self)
		local attached, locked = PetBattleTeamsSettings:GetFrameInfo()
		if not attached and not locked then
			
			frame:SetResizable(true) 
			frame:StartSizing() 
		end
	end)
	frame.resizeButton:SetScript("OnDragStop",function(self)
		frame:StopMovingOrSizing()
		frame:SetResizable(false) 
		
		frame:SetSize(frame:GetWidth(), ROW_HEIGHT*PetBattleTeamsUI:GetMaxRows()+90)
	end)
	return frame
end

PetBattleTeamsUI.CreateAddTeamButton = function(self,name,parent)
	local button = CreateFrame("Button", name, parent)
	button:SetSize(40,40)
	
	button.icon = button:CreateTexture(name.."Icon","ARTWORK")
	button.icon:SetTexture("Interface\\GuildBankFrame\\UI-GuildBankFrame-NewTab")
	button.icon:SetAllPoints(button)
	button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")
	button:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
	button:SetScript("OnEnter",function(self)
		GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT")
		GameTooltip:ClearLines()
		GameTooltip:AddLine("Click to add a new team")
		if #PetBattleTeamsSettings.teams >= MAX_TEAMS then
			GameTooltip:AddLine("At maximum number of teams",1,0,0,true)
		end
		GameTooltip:Show()
	end)
	button:SetScript("OnLeave",function(self)
		GameTooltip:Hide()
	end)
	
	button:SetScript("OnClick",function(self) 
		PetBattleTeams:CreateNewTeam()
		PetBattleTeamsUI:UpdateAllFrames()
	end)
	
	return button
end

PetBattleTeamsUI.CreatePetButton = function(self,team,petIndex,parent)
	
	local parentName = "PetTeam"..team.."Button"..petIndex
	local button = CreateFrame("Button", parentName, parent, "PetBattleMiniUnitFrameAlly")
	button:EnableMouse(true)
	button.team = team
	button.petIndex = petIndex
	
	-- this is wrong but works and is to ingrained to fix.
	button:SetSize(40,40)
	button.BorderAlive:SetSize(50,50)
	button.BorderDead:SetSize(50,50)
	-- end bad code
	
	button.healthBarWidth = 37
	
	button.ActualHealthBar.color = {button.ActualHealthBar:GetVertexColor()}
	
	button.rarityGlow = button:CreateTexture(parentName.."Glow","ARTWORK") 
	button.rarityGlow:SetTexture("Interface\\Buttons\\UI-ActionButton-Border")
	button.rarityGlow:SetBlendMode("ADD")
	button.rarityGlow:ClearAllPoints()
	button.rarityGlow:SetDrawLayer("ARTWORK", 0)
	button.rarityGlow:SetWidth(button:GetWidth() * 1.6)
	button.rarityGlow:SetHeight(button:GetHeight() * 1.4)
	button.rarityGlow:SetPoint("CENTER", button.Icon, "CENTER", 0, 4)
	
	button.level = button:CreateFontString(parentName.."Level","OVERLAY","GameFontNormalSmall")
	button.level:SetJustifyH("RIGHT")
	button.level:SetText("XX")
	button.level:SetSize(0,0)
	button.level:SetFont("Fonts\\FRIZQT__.TTF",11,"OUTLINE")
	button.level:SetPoint("BOTTOMRIGHT",button.Icon,"BOTTOMRIGHT",0,10)
	
	button.selected = button:CreateTexture(parentName.."Selected","OVERLAY") 
	button.selected:SetSize(36,36)
	button.selected:SetTexture("Interface\\PetBattles\\PetJournal")
	button.selected:SetTexCoord(0.11328125,0.16210938,0.02246094,0.04687500)
	button.selected:SetPoint("CENTER",button,"LEFT",0,0)
	button.selected:Hide()
	
	button:SetScript("OnLoad",nil)
	button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	button:SetScript("OnClick", function(self,button) 
		
		local info, petID = GetCursorInfo()
		
		if ( button == "RightButton" ) then
				StaticPopup_Show("PBT_TEAM_DELETE",self.team,nil,self.team)
		elseif button == "LeftButton" then
			if info and info == "battlepet" then
				local success = false
				if PetBattleTeamsUI.CursorStruct and PetBattleTeamsUI.CursorStruct.sourceTeam then
					success = PetBattleTeams:UpdateTeamSwapPets(self.team,self.petIndex, PetBattleTeamsUI.CursorStruct.sourceTeam,PetBattleTeamsUI.CursorStruct.soucePetIndex)
				else
					success = PetBattleTeams:UpdateTeamNewPet(petID,self.team,self.petIndex) -- UpdateTeamNewPet =function(self,petID,team,petIndex)
				end
				if success then
					ClearCursor()
					PetBattleTeamsUI:CursorEmpty()
				end
			else
				PetBattleTeams:SetTeam(self.team)
			end
			
			if IsShiftKeyDown() then
				local petLink = C_PetJournal.GetBattlePetLink(self.petID)
				ChatEdit_InsertLink(petLink)
			end
			
			PetBattleTeamsUI:UpdateAllFrames()
		end
	end)
	
	button:SetScript("OnReceiveDrag", function(self,button) 
		local info, petID = GetCursorInfo()
		if info and info == "battlepet" then
			local success = false
			if PetBattleTeamsUI.CursorStruct and PetBattleTeamsUI.CursorStruct.sourceTeam then
				success = PetBattleTeams:UpdateTeamSwapPets(self.team,self.petIndex, PetBattleTeamsUI.CursorStruct.sourceTeam, PetBattleTeamsUI.CursorStruct.soucePetIndex)
			else
				success = PetBattleTeams:UpdateTeamNewPet(petID,self.team,self.petIndex) -- UpdateTeamNewPet =function(self,petID,team,petIndex)
			end
			if success then
				ClearCursor()
				PetBattleTeamsUI:CursorEmpty()
				PetBattleTeamsUI:UpdateAllFrames()
			end
		end
		
	end)
	
	
	
	button:RegisterForDrag("LeftButton")
	
	
	
	button:SetScript("OnDragStart", function(self,button)
		C_PetJournal.PickupPet(self.petID, false)
		PetBattleTeamsUI.CursorStruct = {}
		PetBattleTeamsUI.CursorStruct.sourceTeam = self.team
		PetBattleTeamsUI.CursorStruct.soucePetIndex = self.petIndex
		PetBattleTeamsUI:UpdateAllFrames()
	end)

	button:SetScript("OnEnter",function(self) 
		if C_PetJournal.GetPetInfoByPetID(self.petID) then
			local tooltip = PetBattleTeamsUI.tooltip
			tooltip:SetParent(self)
			tooltip:SetFrameStrata("TOOLTIP")
			tooltip:ClearAllPoints()
			
			tooltip:SetPoint("TOPLEFT", self, "BOTTOMRIGHT", 0, 0)
			local left, bottom, width, height = tooltip:GetBoundsRect()
			
			if left + width > GetScreenWidth() then 
				tooltip:ClearAllPoints()
				tooltip:SetPoint( "TOPRIGHT", self, "BOTTOMLEFT", 0, 0)
			end
			if bottom < 0 then
				tooltip:ClearAllPoints()
				tooltip:SetPoint("BOTTOMRIGHT", self, "TOPLEFT", 0, 0)
			end

			PetBattleTeamsUI:TooltipSetUnit(PetBattleTeamsUI.tooltip, self.petID, self.team, self.petIndex)
			tooltip:Show()
		end
	end)
	
	button:SetScript("OnLeave",function(self)
		if ( PetBattleTeamsUI.tooltip:GetParent() == self ) then
			PetBattleTeamsUI.tooltip:Hide()
		end
	end)
	button:UnregisterAllEvents()
	return button
end

PetBattleTeamsUI.CreatePetUnitFrames = function(self,parent)
	local unitFrames = {}
	for team=1,MAX_ROWS do 
		local buttons = {}
		for petIndex =1, PETS_PER_TEAM do 
			button = self:CreatePetButton(team,petIndex,parent)
			
			if petIndex==1 and team == 1 then
				
				button:SetPoint("TOPLEFT", parent ,"TOPLEFT",20,-30)
				--button:SetPoint("TOPLEFT", parent ,"TOPLEFT",17,)
			elseif petIndex == 1 and team > 1 then
				button:SetPoint("TOPLEFT","PetTeam"..(team-1).."Button"..petIndex,"BOTTOMLEFT",0,-4)
			else
				button:SetPoint("LEFT","PetTeam"..team.."Button"..petIndex-1,"RIGHT",5,0)
			end
			
			table.insert(buttons,button)
		end
		table.insert(unitFrames,buttons)
	end
	
	return unitFrames
end

PetBattleTeamsUI.CreateScrollBar = function(self,name,parent)
	frame = CreateFrame("ScrollFrame",name,parent,"FauxScrollFrameTemplate")
	frame:ClearAllPoints()
	frame:SetPoint("TOPLEFT", 0, -30)
	frame:SetPoint("BOTTOMRIGHT", -30, 60)
	frame:EnableMouse(false)
	frame:SetScript("OnVerticalScroll",function(self,offset)
		scrollBar = _G[PetBattleTeamsUI.scrollBar:GetName() .. "ScrollBar"];
		self.offset = math.floor(offset / ROW_HEIGHT + 0.5)
		scrollBar:SetValue(offset)
		PetBattleTeamsUI:UpdateScrollBar()
	end)
	frame:SetScript("OnShow",function(self) 
		PetBattleTeamsUI:UpdateScrollBar()
	end)
	return frame
end

PetBattleTeamsUI.TeamVisible = function(self,team)
	if #PetBattleTeamsSettings.teams <= 4 then 
		return true;
	elseif self.scrollBar.offset + self:GetMaxRows() < team or self.scrollBar.offset >= team  then
		return false
	end
	return true
end

PetBattleTeamsUI.ScrollToTeam =function(self,team)
	if not self:TeamVisible(team) then
		scrollBar = _G[PetBattleTeamsUI.scrollBar:GetName() .. "ScrollBar"];
		local scrollPosition = 0
		
		if team > 0 then
			scrollPosition = (team /  #PetBattleTeamsSettings.teams) * math.ceil(self.scrollBar:GetVerticalScrollRange())
		end
		
		scrollBar:SetValue(scrollPosition);
		self:UpdateAllFrames()
	end
end

PetBattleTeamsUI.GetMaxRows =function(self)
	local extraSpace = 80
	local extraRow = 0
	local attached = PetBattleTeamsSettings:GetFrameInfo()
	if attached then
		extraSpace = 0 
		extraRow = 1
	end
	local maxRows = math.floor(((self.mainFrame:GetHeight()-extraSpace)/ROW_HEIGHT))-extraRow
	return maxRows
end

PetBattleTeamsUI.UpdateScrollBar = function(self,msg)
	
	if self.scrollBar then
		
		local maxRows = self:GetMaxRows();
		local maxValue = #PetBattleTeamsSettings.teams
		local offset = FauxScrollFrame_GetOffset(self.scrollBar)
		local row = self.mainFrame.petUnitFrames
		
		PetBattleTeamsUI.mainFrame.GotoTeamButton:SetShown(not PetBattleTeamsUI:TeamVisible(PetBattleTeamsSettings.selected))
		FauxScrollFrame_Update(self.scrollBar, maxValue, maxRows, ROW_HEIGHT)	
		
		for i = 1, maxRows do
			local value = i + offset
			if value <= maxValue then
				
				for j=1,PETS_PER_TEAM do
					
					PetBattleTeamsUI:UpdatePetButton(row[i][j],value,j,msg)
				end
			else
				for j=1,PETS_PER_TEAM do
					row[i][j]:Hide()
				end
			end
		end
		
		for i=maxRows+1,MAX_ROWS do
			for j=1,PETS_PER_TEAM do
				if row[i] and row[i][j] then
					row[i][j]:Hide()
				end
			end
		end
	end
end

PetBattleTeamsUI.CreateUI = function(self)	
	
	if not self.tooltip then self.tooltip = self:CreateTooltip("PetBattleTeamTooltip")  end
	if not self.mainFrame then 
		self.mainFrame = self:CreateParentFrame("PetBattleTeamBackgroundFrame")
		self.mainFrame:SetParent("UIParent")
		local attached, locked, x,y, height = PetBattleTeamsSettings:GetFrameInfo()
		self.mainFrame:SetPoint("TOPLEFT",UIParent,"TOPLEFT",x,y)
		self.mainFrame:SetSize(self.mainFrame:GetWidth(), height);
		
	end
	if not self.scrollBar then
		self.scrollBar = self:CreateScrollBar("PetBattleTeamsScroll",self.mainFrame)
		PetBattleTeamsUI:UpdateScrollBar()
	end
end



PetBattleTeamsUI.PickupPetHook = function(petID,isWild)
	PetBattleTeamsUI.PetOnCursor = petID
	PetBattleTeamsUI.CursorStruct = nil
	PetBattleTeamsUI:UpdateAllFrames()
end


hooksecurefunc(C_PetJournal,"PickupPet",PetBattleTeamsUI.PickupPetHook)
--hooksecurefunc("ClearCursor",PetBattleTeamsUI.ClearCursorHook)
